<?php
/**
 * Conclude Poll
 *
 * This script concludes voting on a poll (sets the end date to the
 * current time).
 *
 * $Horde: scry/conclude.php,v 1.9 2005/01/03 14:35:27 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('SCRY_BASE', dirname(__FILE__));
require_once SCRY_BASE . '/lib/base.php';
require_once 'Horde/Variables.php';

/**
 * Conclude Class
 *
 * This form is for confirming the closing of a poll.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.9 $
 * @package Scry
 */
class ConcludeForm extends Horde_Form {

    var $_useFormToken = true;

    function ConcludeForm(&$vars, $inactive = false)
    {
        parent::Horde_Form($vars, sprintf(_("Conclude Poll: %s"), $vars->get('poll_title')));

        $this->setButtons(array(_("Conclude"), _("Cancel")));

        $this->addHidden('', 'poll_id', 'int', true);
        $this->addHidden('', 'url', 'text', false);
        $this->addVariable(sprintf(_("Here are the current results for '%s'"), $vars->get('poll_question')), 'poll_question', 'header', false);
        $this->addVariable($vars->get('results'), 'results', 'description', false, true);
        $this->addVariable(sprintf(_("Do you want to stop voting on this poll now? If you do, the results will be processed and no one will be able to enter new votes or change their vote. The poll will remain available for later viewing and analysis. Conclude '%s'?"), $vars->get('poll_title')), 'conclude_confirm', 'description', true);
    }

}

/* Deny access to users who aren't poll admins. */
if (!Auth::isAdmin('scry:admin')) {
    Horde::authenticationFailureRedirect();
}

$renderer = &new Horde_Form_Renderer();

$done = false;
$vars = &Variables::getDefaultVariables();
$poll = $scry->getPolls($vars->get('poll_id'));
foreach ($poll as $varname => $value) {
    $vars->add($varname, $value);
}
$form_submit = $vars->get('submitbutton');

/* Only bother getting these if not yet submitted. */
if (!$form_submit) {
    $votes = $scry->getVotes($vars->get('poll_id'));
    $vars->set('results', Scry::getResults($votes));
}

$form = &Horde_Form::singleton('ConcludeForm', $vars);

if ($form_submit == _("Conclude")) {
    $form->validate($vars);

    if ($form->isValid()) {
        $success = $scry->concludePoll($vars->get('poll_id'));
        if (is_a($success, 'PEAR_Error')) {
            $notification->push(sprintf(_("Error concluding poll. %s."), $conclude->getMessage()), 'horde.error');
        } else {
            $notification->push(_("Poll concluded. Voting is now closed."), 'horde.success');
            $done = true;
        }
    }
} elseif (!empty($form_submit)) {
    $notification->push(_("Poll has not been concluded."), 'horde.message');
    $done = true;
}

/* If finished redirect back to calling url or default to poll
 * list. */
if ($done) {
    $url = $vars->get('url');
    if (!$url) {
        $url = Horde::applicationUrl('polls.php', true);
    }
    header('Location: ' . $url);
    exit;
}

/* Get the conlude form HTML. */
$main = Util::bufferOutput(array($form, 'renderActive'), $renderer, $vars, 'conclude.php', 'post');

/* Set up the template variables. */
$template = &new Horde_Template();
$template->set('menu', Scry::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));
$template->set('main', $main);

/* Output the page. */
require SCRY_TEMPLATES . '/common-header.inc';
echo $template->fetch(SCRY_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
