<?php
/**
 * Delete Poll
 *
 * This script deletes a poll and all of its voting logs.
 *
 * $Horde: scry/delete.php,v 1.23 2005/01/03 14:35:27 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('SCRY_BASE', dirname(__FILE__));
require_once SCRY_BASE . '/lib/base.php';
require_once 'Horde/Variables.php';

/**
 * Delete Class
 *
 * This form is for confirming the deletion of a poll.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.23 $
 * @package Scry
 */
class DeleteForm extends Horde_Form {

    function DeleteForm(&$vars, $inactive = false)
    {
        parent::Horde_Form($vars, sprintf(_("Delete Poll: %s"), $vars->get('poll_title')));

        $this->setButtons(array(_("Delete"), _("Cancel")));

        $this->addHidden('', 'poll_id', 'int', true);
        $this->addHidden('', 'url', 'text', false);
        $this->addVariable(sprintf(_("Voting results for: %s"), $vars->get('poll_question')), 'poll_question', 'header', false);
        $this->addVariable($vars->get('results'), 'results', 'description', false, true);
        $this->addVariable(sprintf(_("If you delete this poll, the voting logs will no longer be available. Do you really want to delete '%s'?"), $vars->get('poll_title')), 'delete_confirm', 'description', true);
    }

}

/* Deny access to users who aren't poll admins. */
if (!Auth::isAdmin('scry:admin')) {
    Horde::authenticationFailureRedirect();
}

$renderer = &new Horde_Form_Renderer();

$done = false;
$vars = &Variables::getDefaultVariables();
$poll = $scry->getPolls($vars->get('poll_id'));
foreach ($poll as $varname => $value) {
    $vars->add($varname, $value);
}
$form_submit = $vars->get('submitbutton');

/* Only bother getting these if not yet submitted. */
if (!$form_submit) {
    $votes = $scry->getVotes($vars->get('poll_id'));
    $vars->set('results', Scry::getResults($votes));
}

$formdelete = &Horde_Form::singleton('DeleteForm', $vars);

if ($form_submit == _("Delete")) {
    $formdelete->validate($vars);

    if ($formdelete->isValid()) {
        $delete = $scry->deletePoll($vars->get('poll_id'));
        if (is_a($delete, 'PEAR_Error')) {
            $notification->push(sprintf(_("Error deleting poll. %s."), $delete->getMessage()), 'horde.error');
        } else {
            $notification->push(_("Poll deleted."), 'horde.success');
            $done = true;
        }
    }
} elseif (!empty($form_submit)) {
    $notification->push(_("Poll has not been deleted."), 'horde.message');
    $done = true;
}

/* If finished redirect back to calling url or default to poll
 * list. */
if ($done) {
    $url = $vars->get('url');
    if (!$url) {
        $url = Horde::applicationUrl('polls.php', true);
    }
    header('Location: ' . $url);
    exit;
}

/* Get the delete form HTML. */
$main = Util::bufferOutput(array($formdelete, 'renderActive'), $renderer, $vars, 'delete.php', 'post');

/* Set up the template variables. */
$template = &new Horde_Template();
$template->set('menu', Scry::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));
$template->set('main', $main);

/* Output the page. */
require SCRY_TEMPLATES . '/common-header.inc';
echo $template->fetch(SCRY_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
