<?php
/**
 * The Scry script to edit polls.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * $Horde: scry/edit.php,v 1.37 2005/01/03 14:35:27 jan Exp $
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('SCRY_BASE', dirname(__FILE__));
require_once SCRY_BASE . '/lib/base.php';
require_once 'Horde/Form/Action.php';
require_once 'Horde/Variables.php';

/**
 * Poll editing class.
 *
 * @package Scry
 */
class PollForm extends Horde_Form {

    function PollForm(&$vars, $title)
    {
        global $conf, $num_choices;

        parent::Horde_Form($vars, $title);

        $this->setButtons(_("Save"), true);

        $this->addHidden('', 'poll_id', 'int', false);
        $this->addHidden('', 'old_num_choices', 'int', false);

        /* Set up the choices field. */
        $choices = array();
        for ($i = 2; $i <= $conf['polls']['maxchoices']; $i++) {
            $choices[$i] = $i;
        }
        $v = &$this->addVariable(_("Number of choices"), 'num_choices', 'enum', false, false, null, array($choices));
        $v->setDefault('2');
        $action = &Horde_Form_Action::factory('submit');
        $v->setAction($action);
        $v->setHelp('edit-numchoices');

        $this->addVariable(_("Title (short description)"), 'poll_title', 'text', true);
        $this->addVariable(_("Enter question"), 'poll_question', 'longtext', true, false, null, array(4, 40));

        /* Set up the required number of choice fields. */
        $vars->set('old_num_choices', $num_choices);
        $num_choices = $vars->get('num_choices');
        if (empty($num_choices)) {
            /* If not yet set default to 2, and make sure that
             * old_num_choices defaults also. */
            $num_choices = 2;
            $vars->set('old_num_choices', 2);
        }
        for ($i = 0; $i < $num_choices; $i++) {
            $this->addVariable(sprintf(_("Choice number %s"), $i + 1), 'choices_' . $i, 'text', true);
        }

        $v = &$this->addVariable(_("Start date"), 'poll_start', 'monthdayyear', false, false, null, array(null, date('Y') + 3, true, 5));
        $v->setHelp('edit-startdate');

        $v = &$this->addVariable(_("End date"), 'poll_end', 'monthdayyear', false, false, null, array(null, date('Y') + 3, true, 5));
        $v->setHelp('edit-enddate');

        $v = &$this->addVariable(_("Can multiple answers be chosen?"), 'poll_multiple', 'radio', false, false, null, array(array(0 => _("Vote for one choice"),
                                                                                                                                 1 => _("Vote for one or more choices"))));
        $v->setDefault(0);
        $v->setHelp('edit-multiple');

        $v = &$this->addVariable(_("Is this poll active?"), 'poll_enabled', 'radio', false, false, null, array(array(1 => _("Poll is active"),
                                                                                                                     0 => _("Poll is disabled (hidden)"))));
        $v->setDefault(1);
        $v->setHelp('edit-enabled');

        /* Groups. */
        require_once 'Horde/Group.php';
        $groups = &Group::singleton();
        $mygroups = $groups->getGroupMemberships(Auth::getAuth());
        $mygroups = array('' => _("All Users")) + $mygroups;
        $this->addVariable(_("Is this poll for a specific group?"), 'poll_group', 'enum', false, false, null, array($mygroups));

        /* Fetch the $conf allowed identifiers. */
        $identifiers = Scry::getIdentifiers();
        $v = &$this->addVariable(_("Use identifiers"), 'poll_identifier', 'multienum', true, false, null, array($identifiers, count($identifiers)));
        $v->setDefault(array_keys($identifiers));
        $v->setHelp('edit-identifiers');
    }

}

/* Deny access to users who aren't poll admins. */
if (!Auth::isAdmin('scry:admin')) {
    $notification->push(_("You are not authorised for this action."), 'horde.warning');
    Horde::authenticationFailureRedirect();
}

/* Set up the form variables and the form. */
$vars = &Variables::getDefaultVariables();

/* Set up some variables. */
$changed_num = false;
$formname = $vars->get('formname');
$poll_id = $vars->get('poll_id');
$old_num_choices = $vars->get('old_num_choices');
$url = $vars->get('url');

/* If editing a poll and not yet submitted. */
if ($poll_id && !$formname) {
    /* Fetch the poll details. */
    $poll = $scry->getPolls($poll_id);
    $enabled = Util::getFormData('enabled');
    $title = sprintf(_("Edit %s"), $poll['poll_title']);

    /* If not toggling state, allow editing only polls which have not
     * started. */
    if (is_null($enabled) && !$scry->isEditable($poll_id)) {
        $notification->push(_("This poll is not editable."), 'horde.warning');
        if (empty($url)) {
            $url = Util::addParameter(Horde::applicationUrl('view.php', true), 'poll_id', $poll_id, false);
        }
        header('Location: ' . $url);
        exit;
    } elseif (!is_null($enabled)) {
        $scry->enablePoll($poll_id, $enabled);
        $notification->push(sprintf(_("Poll '%s' has been %s."), $poll['poll_title'], ($enabled ? _("enabled") : _("disabled"))) , 'horde.success');
        if (empty($url)) {
            $url = Util::addParameter(Horde::applicationUrl('view.php', true), 'poll_id', $poll_id, false);
        }
        header('Location: ' . $url);
        exit;
    }

    /* Make sure that the poll identifiers default correctly. */
    $poll['poll_identifier'] = Scry::getIdentifierArray($poll['poll_identifier']);

    /* Remap the choices. */
    $i = 0;
    foreach ($poll['choices'] as $choice) {
        $poll['choices_' . $i] = $choice;
        $i++;
    }
    $poll['num_choices'] = $num_choices = $i;

    $vars = &new Variables($poll);
} else {
    $title = _("New Poll");
}

$num_choices = $vars->get('num_choices');

if ($num_choices != $old_num_choices && $formname) {
    $changed_num = true;
    $notification->push(_("Changed number of choices."), 'horde.message');
}

/* Set up the form object. */
$formedit = &Horde_Form::singleton('PollForm', $vars, $title);

/* Validate/process form only if formname present and number of
 * choices not changed. */
if ($formname && !$changed_num) {
    $formedit->validate($vars);

    if ($formedit->isValid()) {
        $formedit->getInfo($vars, $info);
        $poll_id = $scry->savePoll($info);
        if (is_a($poll_id, 'PEAR_Error')) {
            $notification->push(sprintf(_("There was an error saving the poll. %s"), $poll_id->getMessage()), 'horde.error');
        } else {
            $notification->push(sprintf(_("The poll '%s' has been saved."), $info['poll_title']), 'horde.success');
            $url = Util::addParameter(Horde::applicationUrl('view.php', true), 'poll_id', $poll_id, false);
            header('Location: ' . $url);
            exit;
        }
    }
}

/* Render the form. */
$renderer = &new Horde_Form_Renderer();
$main = Util::bufferOutput(array($formedit, 'renderActive'), $renderer, $vars, 'edit.php', 'post');

$template = &new Horde_Template();
$template->set('menu', Scry::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));
$template->set('main', $main);

require SCRY_TEMPLATES . '/common-header.inc';
echo $template->fetch(SCRY_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
