<?php

$block_name = _("Show Poll");

/**
 * Scry Poll View Block Class
 *
 * This file provides an api to include Scry polls in any other Horde app
 * through the Horde_Blocks, by extending the Horde_Blocks class.
 *
 * $Horde: scry/lib/Block/poll.php,v 1.43 2005/01/03 14:35:28 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.43 $
 * @package Horde_Block
 */
class Horde_Block_scry_poll extends Horde_Block {

    var $_app = 'scry';

    function Horde_Block_scry_poll($params)
    {
        parent::Horde_Block($params);
        @define('SCRY_BASE', dirname(__FILE__) . '/../..');
        require_once SCRY_BASE . '/lib/Scry.php';
    }

    function _params()
    {
        require_once SCRY_BASE . '/lib/Driver.php';
        $scry = &Scry_Driver::singleton();

        $params['poll_id'] = array('name' => _("Poll"),
                                   'type' => 'enum',
                                   'values' => array());

        $votable_polls = $scry->getPolls(null, 'open');
        foreach ($votable_polls as $poll_id => $poll_val) {
            $params['poll_id']['values'][$poll_id] = $poll_val['poll_title'];
        }
        $params['poll_id']['values']['_latest'] = _("Always show latest");

        return $params;
    }

    function _title()
    {
        require_once SCRY_BASE . '/lib/Driver.php';
        $scry = &Scry_Driver::singleton();

        $poll = $this->_getPollData($this->_params['poll_id']);
        if (!count($poll)) {
            return _("Poll not found");
        }

        $url = Util::addParameter(Horde::applicationUrl('vote.php', true), 'poll_id', $poll['poll_id']);
        return Horde::link($url, $poll['poll_title'], 'header') . $poll['poll_title'] . '</a>';
    }

    function _content()
    {
        require_once SCRY_BASE . '/lib/base.php';
        require_once SCRY_BASE . '/lib/Vote.php';
        global $prefs, $notification, $scry;

        $poll = $this->_getPollData($this->_params['poll_id']);
        if (!count($poll)) {
            return _("Poll not found");
        }

        /* Set up the form variables. */
        require_once 'Horde/Variables.php';
        $vars = &Variables::getDefaultVariables();
        $form_submitted = ($vars->get('formname') == 'voteform');

        /* Check if form has been submitted for a voteable poll. */
        $is_votable = $scry->isVotable($poll);
        if ($form_submitted && $is_votable) {
            $formvote = &Horde_Form::singleton('VoteForm', $vars);
            if ($formvote->validate($vars)) {
                $formvote->getInfo($vars, $info);
                $results = $scry->logVote($info);
                if (is_a($results, 'PEAR_Error')) {
                    $notification->push(sprintf(_("Voting failed. %s"), $results->getMessage()), 'horde.error');
                } else {
                    $notification->push(_("Thank you for your vote."), 'horde.success');
                    $url = Horde::selfURL(true, false, true);
                    header('Location: ' . $url);
                    exit;
                }
            }
        }

        /* Set up the renderer. */
        $vars = &new Variables($poll);
        $formvote = &Horde_Form::singleton('VoteForm', $vars, '');
        $renderer = &$formvote->getRenderer();

        if (!$is_votable) {
            $formvote->addVariable('', 'results', 'html', false);

            /* Get the current voting results */
            $votes = $scry->getVotes($poll['poll_id']);
            $vars->set('results', Scry::getResults($votes));
            $main = Util::bufferOutput(array($formvote, 'renderInactive'), $renderer, $vars);
        } else {
            $main = Util::bufferOutput(array($formvote, 'renderActive'), $renderer, $vars, Horde::selfURL(true), 'post');
        }

        /* Set the main body to the template. */
        $template = &new Horde_Template();
        $template->set('main', $main);

        $html = $template->fetch(SCRY_TEMPLATES . '/block/poll.html');

        return $html;
    }

    function _getPollData($poll_id)
    {
        static $polls;

        if (!isset($polls[$poll_id])) {
            require_once SCRY_BASE . '/lib/Driver.php';
            $scry = &Scry_Driver::singleton();

            if ($poll_id == '_latest') {
                $polls[$poll_id] = current($scry->getPolls(null, 'open', 0, 1));
            } else {
                $polls[$poll_id] = $scry->getPolls($poll_id);
            }
        }

        return $polls[$poll_id];
    }

}
