<?php
/**
 * Scry_Driver Class
 *
 * $Horde: scry/lib/Driver.php,v 1.27 2005/01/03 14:35:27 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.27 $
 * @package Scry
 */
class Scry_Driver {

    var $_params;

    function Scry_Driver($params)
    {
        $this->_params = $params;
    }

    function savePoll(&$info)
    {
        $info['poll_identifier'] = array_sum($info['poll_identifier']);

        return $this->_savePoll($info);
    }

    function logVote($vote)
    {
        $poll = $this->getPolls($vote['poll_id']);
        if (is_a($poll, 'PEAR_Error')) {
            return $poll;
        }

        if ($poll['poll_identifier'] > 0 && !$this->isVotable($poll)) {
            return PEAR::raiseError(_("You can not vote for this poll."));
        }

        if (!$poll['poll_multiple'] && is_array($vote['choice'])) {
            return PEAR::raiseError(_("You are only allowed to vote for one choice on this poll."));
        }

        /* If using cookies, add to them the new poll. */
        if ((int)$poll['poll_identifier'] & SCRY_ID_COOKIE) {
            $voted_polls = isset($_COOKIE['scry_voted']) ? $_COOKIE['scry_voted'] : '';
            if (empty($voted_polls)) {
                $voted_polls = $vote['poll_id'];
            } else {
                $voted_polls .= '|' . $vote['poll_id'];
            }
            $e = setcookie('scry_voted', $voted_polls, time() + 31536000, $GLOBALS['conf']['cookie']['path'], $GLOBALS['conf']['cookie']['domain']);
        }

        /* Set up all the required identifiers. */
        $poll['poll_identifier'] = (int)$poll['poll_identifier'];
        $vote['vote_ip'] = ($poll['poll_identifier'] & SCRY_ID_IP) ? $_SERVER['REMOTE_ADDR'] : null;
        $vote['vote_username'] = ($poll['poll_identifier'] & SCRY_ID_AUTH) ? Auth::getAuth() : null;
        $vote['vote_identifier'] = ($poll['poll_identifier'] & SCRY_ID_CUSTOM) ? $vote['vote_identifier'] : null;
        $vote['vote_confirmation'] = ($poll['poll_identifier'] & SCRY_ID_CONFIRM) ? md5(mt_rand()) : null;

        return $this->_logVote($poll, $vote);
    }

    function isVotable($poll)
    {
        static $votable = array();
        if (isset($votable[$poll['poll_id']])) {
            return $votable[$poll['poll_id']];
        }

        $votable[$poll['poll_id']] = true;

        /* Check if the poll is enabled and within the time limits. */
        if (!$this->isOpen($poll)) {
            $votable[$poll['poll_id']] = false;
        } elseif (((int)$poll['poll_identifier'] & SCRY_ID_COOKIE) &&
                  isset($_COOKIE['scry_voted'])) {
            $polls_voted = explode('|', $_COOKIE['scry_voted']);
            if (in_array($poll['poll_id'], $polls_voted)) {
                $votable[$poll['poll_id']] = false;
            }
        } elseif ($poll['poll_identifier'] < SCRY_ID_IP) {
            /* No identifier that requires backend checking. */
            $votable[$poll['poll_id']] = true;
        } else {
            /* We need to check from the backend */
            $votable[$poll['poll_id']] = !$this->hasVoted($poll);
        }

        return $votable[$poll['poll_id']];
    }

    function isOpen($poll)
    {
        /* Check if the poll is enabled and within the time limits. */
        if (!$poll['poll_enabled'] ||
            (!is_null($poll['poll_start']) && $poll['poll_start'] > time()) ||
            (!is_null($poll['poll_end']) && $poll['poll_end'] < time())) {
            return false;
        }

        return true;
    }

    /**
     * Returns the editable status of the current poll.
     *
     * @return boolean  Whether the current poll is editable or not.
     */
    function isEditable($poll_id, $votes = null)
    {
        static $is_editable = array();
        if (!isset($is_editable[$poll_id])) {
            if (is_null($votes)) {
                $votes = $this->getVotes($poll_id);
            }
            $is_editable[$poll_id] = array_sum($votes) ? false : true;
        }
        return $is_editable[$poll_id];
    }

    /**
     * Attempts to return a concrete Driver instance based on $driver.
     *
     * @param string $driver The type of concrete Driver subclass to return.
     *                       This is based on the storage driver ($driver). The
     *                       code is dynamically included.
     * @param optional array $params A hash containing any additional
     *                               configuration or connection parameters a
     *                               subclass might need.
     *
     * @return object Driver The newly created concrete Driver instance, or false
     *         on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Scry_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("No such backend '%s' found"), $driver)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete Scry_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Scry_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Scry_Driver::singleton()
     *
     * @param string    $driver     The type of concrete Scry_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Scry_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Scry_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
