<?php
/**
 * Scry_Driver_sql Class
 *
 * $Horde: scry/lib/Driver/sql.php,v 1.50 2005/01/03 14:35:28 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @author  Marko Djukic <marko@oblo.com>
 * @version $Revision: 1.50 $
 * @package Scry
 */
class Scry_Driver_sql extends Scry_Driver {

    /**
     * Database object.
     *
     * @var DB $_db
     */
    var $_db;

    function Scry_Driver_sql($params)
    {
        parent::Scry_Driver($params);
        $this->initialise();
    }

    function _savePoll(&$info)
    {
        if (empty($info['poll_id'])) {
            $info['poll_id'] = $this->_db->nextId($this->_params['table']['polls']);
            if (is_a($info['poll_id'], 'PEAR_Error')) {
                Horde::logMessage($info['poll_id'], __FILE__, __LINE__, PEAR_LOG_ERR);
                return $info['poll_id'];
            }
            $sql = 'INSERT INTO';
            $wsql = '';
        } else {
            $sql = 'UPDATE';
            $wsql = sprintf(' WHERE poll_id = %s', (int)$info['poll_id']);
        }

        if ($sql == 'UPDATE') {
	        $sql_text = '%s %s SET poll_id = %s, poll_title = %s, poll_question = %s, poll_group = %s, poll_start = %s, poll_end = %s, poll_multiple = %s, poll_enabled = %s, poll_identifier = %s %s';
        } else {
	        $sql_text = '%s %s (poll_id, poll_title, poll_question, poll_group, poll_start, poll_end, poll_multiple, poll_enabled, poll_identifier) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)';
        }
        $sql = sprintf($sql_text,
                       $sql,
                       $this->_params['table']['polls'],
                       $info['poll_id'],
                       $this->_db->quote($info['poll_title']),
                       $this->_db->quote($info['poll_question']),
                       (int)$info['poll_group'],
                       $this->_db->quote($info['poll_start']),
                       $this->_db->quote($info['poll_end']),
                       (int)$info['poll_multiple'],
                       (int)$info['poll_enabled'],
                       (int)$info['poll_identifier'],
                       $wsql);
        Horde::logMessage('SQL Query by Scry_Driver_sql::savePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        /* If editing get rid of any previous choices. */
        if ($wsql) {
            $sql = sprintf('DELETE FROM %s %s', $this->_params['table']['choices'], $wsql);
            $this->_db->query($sql);
        }

        /* Save the choices to the db. */
        for ($i = 0; $i < $info['num_choices']; $i++) {
            $choice_id = $this->_db->nextId($this->_params['table']['choices']);
            $sql = sprintf('INSERT INTO %s (choice_id, poll_id, choice_value) VALUES (%s, %s, %s)',
                           $this->_params['table']['choices'],
                           (int)$choice_id,
                           (int)$info['poll_id'],
                           $this->_db->quote($info['choices_' . $i]));
            Horde::logMessage('SQL Query by Scry_Driver_sql::savePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
            $result = $this->_db->query($sql);
            if (is_a($result, 'PEAR_Error')) {
                Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
                return $result;
            }
        }

        return $info['poll_id'];
    }

    /**
     * Fetches the poll data. If a poll id is passed will fetch only
     * one poll, otherwise if the filter is set will fetch a subset of
     * polls. If none of these are set then the whole list of polls
     * will be returned.
     *
     * @param integer $poll_id  (optional) The poll id of the poll to check.
     * @param integer $filter   (optional) Filter for only a certain group
     *                          of polls.
     * @param integer $from     (optional) The poll to start listing at.
     * @param integer $count    (optional) The number of polls to return.
     *
     * @return array            The poll data.
     */
    function getPolls($poll_id = null, $filter = null, $from = 0, $count = 0)
    {
        $sql = $this->_buildPollsQuery($poll_id, $filter, false);

        if ($count) {
            $sql = $this->_db->modifyLimitQuery($sql, $from, $count);
            if (is_a($sql, 'PEAR_Error')) {
                return $sql;
            }
        }

        Horde::logMessage('SQL Query by Scry_Driver_sql::getPolls(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        } elseif ($result->numRows() == 0) {
            return array();
        }

        $poll = array();
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
            /* Skip past any bad data from mismatched choices. */
            if (!$row['poll_id']) {
                continue;
            }
            if (!isset($poll[$row['poll_id']])) {
                $poll[$row['poll_id']] = $row;
                $poll[$row['poll_id']]['poll_identifier'] = (int)$poll[$row['poll_id']]['poll_identifier'];
                unset($poll[$row['poll_id']]['choice_id']);
                unset($poll[$row['poll_id']]['choice_value']);
            }
            $poll[$row['poll_id']]['choices'][$row['choice_id']] = $row['choice_value'];
        }

        /* Get rid of anything we aren't allowed to see. */
        $poll = Scry::permissionsFilter($poll, 'poll');

        /* If asked for one poll, don't bother with the poll_id as keys */
        if (!is_null($poll_id)) {
            return isset($poll[$poll_id]) ? $poll[$poll_id] : array();
        }

        return $poll;
    }

    /**
     * Return the number of polls
     *
     * @param integer $poll_id  (optional) The poll id of the poll to check.
     * @param integer $filter   (optional) Filter for only a certain group
     *                          of polls.
     *
     * @return array            The poll data.
     */
    function countPolls($poll_id = null, $filter = null)
    {
        $sql = $this->_buildPollsQuery($poll_id, $filter, true);
        Horde::logMessage('SQL Query by Scry_Driver_sql::countPolls(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        return $this->_db->getOne($sql);
    }

    /**
     * Creates a SQL query for selecting or counting polls.
     *
     * @param integer $poll_id  (optional) The poll id of the poll to check.
     * @param integer $filter   (optional) Filter for only a certain group
     *                          of polls.
     *
     * @return array            The poll data.
     */
    function _buildPollsQuery($poll_id = null, $filter = null, $count = false)
    {
        $wsql = '';
        if (!is_null($poll_id)) {
            $wsql = sprintf(' WHERE %s.poll_id = %s',
                            $this->_params['table']['polls'],
                            (int)$poll_id);
        } elseif ($filter == 'open') {
            $wsql = sprintf(' WHERE poll_enabled = 1 AND (%1$s >= poll_start OR poll_start IS NULL) AND (%1$s < poll_end OR poll_end IS NULL)', (int)time());
        } elseif ($filter == 'closed') {
            $wsql = sprintf(' WHERE poll_enabled = 1 AND NOT ((%1$s >= poll_start OR poll_start IS NULL) AND (%1$s < poll_end OR poll_end IS NULL))', (int)time());
        } elseif ($filter == 'disabled') {
            $wsql = ' WHERE poll_enabled = 0';
        }

        if ($count) {
            $sql = sprintf('SELECT COUNT(*) FROM %1$s %2$s',
                           $this->_params['table']['polls'],
                           $wsql);
        } else {
            $sql = sprintf('SELECT * FROM %1$s LEFT JOIN %2$s ON %1$s.poll_id = %2$s.poll_id %3$s',
                           $this->_params['table']['polls'],
                           $this->_params['table']['choices'],
                           $wsql);
        }

        return $sql;
    }

    /**
     * Conclude voting on a poll.
     *
     * @param integer $poll_id  The poll id of the poll to conclude.
     *
     * @return boolean          True on success, otherwise PEAR error.
     */
    function concludePoll($poll_id)
    {
        $sql = sprintf('UPDATE %s SET poll_end = %s WHERE poll_id = %s',
                       $this->_params['table']['polls'],
                       time(),
                       (int)$poll_id);
        Horde::logMessage('SQL Query by Scry_Driver_sql::concludePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return true;
    }

    /**
     * Deletes a poll and all of its vote log from the database.
     *
     * @param integer $poll_id  The poll id of the poll to delete.
     *
     * @return boolean          True on success, otherwise PEAR error.
     */
    function deletePoll($poll_id)
    {
        /* Delete the vote log. */
        $sql = sprintf('DELETE FROM %s WHERE poll_id = %s',
                       $this->_params['table']['votes'],
                       (int)$poll_id);
        Horde::logMessage('SQL Query by Scry_Driver_sql::deletePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        /* Delete the choices for this poll. */
        $sql = sprintf('DELETE FROM %s WHERE poll_id = %s',
                       $this->_params['table']['choices'],
                       (int)$poll_id);
        Horde::logMessage('SQL Query by Scry_Driver_sql::deletePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        /* Delete the poll itself. */
        $sql = sprintf('DELETE FROM %s WHERE poll_id = %s',
                       $this->_params['table']['polls'],
                       (int)$poll_id);
        Horde::logMessage('SQL Query by Scry_Driver_sql::deletePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        return true;
    }

    /**
     * Toggles the enabled state of a poll.
     *
     * @param integer $poll_id  The poll id of the poll to toggle.
     * @param integer $enabled  The new enabled state to set.
     *
     * @return boolean          True on success, otherwise PEAR error.
     */
    function enablePoll($poll_id, $enabled)
    {
        $sql = sprintf('UPDATE %s SET poll_enabled = %s WHERE poll_id = %s',
                       $this->_params['table']['polls'],
                       (int)$enabled,
                       (int)$poll_id);
        $enable = $this->_db->query($sql);
        Horde::logMessage('SQL Query by Scry_Driver_sql::enablePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        if (is_a($enable, 'PEAR_Error')) {
            return $enable;
        }

        return true;
    }

    /**
     * Checks to see if a user has already voted this poll.
     *
     * @param integer $poll_id         The poll id of the poll to check.
     * @param string $vote_identifier  The parameter to be used to check
     *                                 the identity of the voter.
     */
    function hasVoted(&$poll, $identifier = null)
    {
        $wsql = array();
        if ($poll['poll_identifier'] & SCRY_ID_IP) {
            $wsql[] = ' vote_ip = ' . $this->_db->quote($_SERVER['REMOTE_ADDR']);
        }
        if ($poll['poll_identifier'] & SCRY_ID_AUTH) {
            $wsql[] = ' vote_username = ' . $this->_db->quote(Auth::getAuth());
        }
        if ($poll['poll_identifier'] & SCRY_ID_CUSTOM) {
            $wsql[] = ' vote_identifier = ' . $this->_db->quote($identifier);
        }

        if (!empty($wsql)) {
            $wsql = sprintf(' AND (%s)', implode(' OR ', $wsql));
        } else {
            $wsql = '';
        }

        $sql = sprintf('SELECT vote_timestamp FROM %s WHERE poll_id = %s %s',
                       $this->_params['table']['votes'],
                       (int)$poll['poll_id'],
                       $wsql);
        Horde::logMessage('SQL Query by Scry_Driver_sql::hasVoted(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $has_voted = $this->_db->getOne($sql);
        if (is_a($has_voted, 'PEAR_Error')) {
            Horde::logMessage($has_voted, __FILE__, __LINE__, PEAR_LOG_ERR);
        } elseif (empty($has_voted)) {
            return false;
        }

        return true;
    }

    function _logVote($poll, $vote)
    {
        if (!is_array($vote['choice'])) {
            $vote['choice'] = array($vote['choice']);
        }

        foreach ($vote['choice'] as $choice) {
            $vote_id = $this->_db->nextId($this->_params['table']['votes']);
            $sql = sprintf('INSERT INTO %s (vote_id, poll_id, choice_id, vote_ip, vote_username, vote_identifier, vote_confirmation, vote_timestamp) VALUES (%s, %s, %s, %s, %s, %s, %s, %s)',
                           $this->_params['table']['votes'],
                           (int)$vote_id,
                           (int)$vote['poll_id'],
                           (int)$choice,
                           $this->_db->quote($vote['vote_ip']),
                           $this->_db->quote($vote['vote_username']),
                           $this->_db->quote($vote['vote_identifier']),
                           $this->_db->quote($vote['vote_confirmation']),
                           (int)time());
            Horde::logMessage('SQL Query by Scry_Driver_sql::logVote(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
            $log_vote = $this->_db->query($sql);
            if (is_a($log_vote, 'PEAR_Error')) {
                Horde::logMessage($log_vote, __FILE__, __LINE__, PEAR_LOG_ERR);
                return $log_vote;
            }
        }

        return true;
    }

    function getVotes($poll_filter = null)
    {
        /* Do not count any votes that haven't been confirmed. */
        $wsql = sprintf(' WHERE %s.vote_confirmation IS NULL',
                        $this->_params['table']['votes']);

        /* Add filtering by poll ids. */
        if (!is_null($poll_filter)) {
            if (is_array($poll_filter)) {
                /* If we were passed an empty array of polls to fetch
                 * votes for, there will be no votes, so just return
                 * an empty array here. */
                if (!count($poll_filter)) {
                    return array();
                }

                $wsql .= sprintf(' AND %s.poll_id IN (%s)',
                                 $this->_params['table']['choices'],
                                 implode(', ', $poll_filter));
            } else {
                $wsql .= sprintf(' AND %s.poll_id = %s',
                                 $this->_params['table']['choices'],
                                 (int)$poll_filter);
            }
        }

        $sql = sprintf('SELECT %1$s.poll_id, choice_value, COUNT(vote_id) AS vote_count
                          FROM %1$s LEFT JOIN %2$s
                          ON %1$s.choice_id = %2$s.choice_id %3$s
                          GROUP BY %1$s.poll_id, %1$s.choice_value, %1$s.choice_id
                          ORDER BY %1$s.poll_id, vote_count DESC',
                        $this->_params['table']['choices'],
                        $this->_params['table']['votes'],
                        $wsql);
        Horde::logMessage('SQL Query by Scry_Driver_sql::getVotes(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $result = $this->_db->query($sql);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        /* Fetch and arrange the votes into a useable array. */
        $votes = array();
        while ($row = $result->fetchRow(DB_FETCHMODE_ASSOC)) {
            $votes[$row['poll_id']][$row['choice_value']] = $row['vote_count'];
        }

        /* If asked for one poll, just return the votes for that
         * poll. */
        if (!is_null($poll_filter) && !is_array($poll_filter)) {
            return $votes[$poll_filter];
        }

        return $votes;
    }

    /**
     * Fetches the full poll voting log for use when exporting the log
     * to a file. The array keys will be set according to locale
     * settings.
     *
     * @param integer $poll_id  The poll id of the poll vote log to fetch.
     *
     * @return array  The vote log data.
     */
    function getVotesLog($poll_id)
    {
        $sql = sprintf('SELECT choice_value AS %1$s, vote_username %2$s, vote_timestamp as %3$s FROM %4$s
                          LEFT JOIN %5$s
                          ON %5$s.choice_id = %4$s.choice_id
                          WHERE %5$s.poll_id = %6$s
                          ORDER BY %4$s.vote_timestamp',
                       $this->_db->quote(_("choice")),
                       $this->_db->quote(_("voted by")),
                       $this->_db->quote(_("time")),
                       $this->_params['table']['votes'],
                       $this->_params['table']['choices'],
                       (int)$poll_id);
        Horde::logMessage('SQL Query by Scry_Driver_sql::savePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $votes = $this->_db->getAll($sql, DB_FETCHMODE_ASSOC);
        if (is_a($votes, 'PEAR_Error')) {
            Horde::logMessage($votes, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $votes;
        }

        return $votes;
    }

    function getWhoChoseWhat($poll_id)
    {
        $wsql = sprintf(' WHERE %s.poll_id = %s ',
                        $this->_params['table']['choices'],
                        (int)$poll_id);
        $sql = sprintf('SELECT choice_value, vote_username FROM %1$s
                          LEFT JOIN %2$s
                          ON %1$s.choice_id = %2$s.choice_id %3$s',
                       $this->_params['table']['choices'],
                       $this->_params['table']['votes'],
                       $wsql);
        Horde::logMessage('SQL Query by Scry_Driver_sql::savePoll(): ' . $sql, __FILE__, __LINE__, PEAR_LOG_DEBUG);
        $raw = $this->_db->getAll($sql);
        if (is_a($raw, 'PEAR_Error')) {
            Horde::logMessage($raw, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $raw;
        }

        $votes = array();
        foreach ($raw as $info) {
            if (!isset($votes[$info[0]])) {
                $votes[$info[0]] = array();
            }
            $votes[$info[0]][] = $info[1];
        }

        return $votes;
    }

    function initialise()
    {
        global $registry;

        Horde::assertDriverConfig($this->_params, 'storage',
            array('phptype', 'hostspec', 'username', 'database'));

        /* Default values for the three database tables. */
        if (!isset($this->_params['password'])) {
            $this->_params['password'] = '';
        }
        if (empty($this->_params['table']['polls'])) {
            $this->_params['table']['polls'] = 'scry_polls';
        }
        if (empty($this->_params['table']['choices'])) {
            $this->_params['table']['choices'] = 'scry_choices';
        }
        if (empty($this->_params['table']['votes'])) {
            $this->_params['table']['votes'] = 'scry_votes';
        }

        /* Connect to the SQL server using the supplied parameters. */
        require_once 'DB.php';
        $this->_db = &DB::connect($this->_params,
                                  array('persistent' => !empty($this->_params['persistent'])));
        if (is_a($this->_db, 'PEAR_Error')) {
            Horde::fatal($this->_db, __FILE__, __LINE__);
        }

        $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

        return true;
    }

}
