<?php
/**
 * The Scry start page showing a list of the available polls.
 *
 * $Horde: scry/polls.php,v 1.39 2005/01/03 14:35:27 jan Exp $
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('SCRY_BASE', dirname(__FILE__));
require_once SCRY_BASE . '/lib/base.php';
require_once 'Horde/UI/Tabs.php';
require_once 'Horde/Variables.php';

$vars = &Variables::getDefaultVariables();

/* Check for changes in the show preference for what kind of polls we
 * want to see. */
if ($vars->exists('show')) {
    $prefs->setValue('show', $vars->get('show'));
} else {
    $vars->set('show', $prefs->getValue('show'));
}

$polls_num = $scry->countPolls(null, $prefs->getValue('show'));
if (is_a($polls_num, 'PEAR_Error')) {
    $notification->push($polls_num, 'horde.error');
    $polls_num = 0;
}
$page = Util::getGet('page', 0);
$perpage = $prefs->getValue('perpage');

$polls = $scry->getPolls(null, $prefs->getValue('show'), $page * $perpage, $perpage);
$votes = $scry->getVotes(is_a($polls, 'PEAR_Error') ? null : array_keys($polls));

if (is_a($polls, 'PEAR_Error')) {
    $notification->push($polls, 'horde.error');
    $polls = array();
} elseif (is_a($votes, 'PEAR_Error')) {
    $notification->push($votes, 'horde.error');
    $votes = array();
} elseif (!count($polls)) {
    switch ($prefs->getValue('show')) {
    case 'open':
        $notification->push(_("There are no open polls at this time."), 'horde.message');
        break;

    case 'closed':
        $notification->push(_("There are no closed polls at this time."), 'horde.message');
        break;

    case 'disabled':
        $notification->push(_("There are no disabled polls at this time."), 'horde.message');
        break;

    default:
        $notification->push(_("There are no polls at this time."), 'horde.message');
        break;
    }
} else {
    foreach ($polls as $poll_id => $poll) {
        $polls[$poll_id]['poll_start'] = Scry::getDate($poll['poll_start'], _("No Start Date"));
        $polls[$poll_id]['poll_end'] = Scry::getDate($poll['poll_end'], _("No End Date"));
        list($polls[$poll_id]['poll_enabled'], $polls[$poll_id]['poll_enabled_class']) = Scry::getStatusLabel($poll['poll_enabled'], $poll['poll_start'], $poll['poll_end']);
        $polls[$poll_id]['links'] = array();

        /* Vote link. */
        if ($scry->isVotable($poll)) {
            $url = Util::addParameter(Horde::applicationUrl('vote.php'), 'poll_id', $poll_id);
            $polls[$poll_id]['links'][] = Horde::link($url, _("Vote"), 'smallheader') . _("Vote") . '</a>';
        }

        /* Allow admins to conclude polls. */
        if (Scry::checkPerms() && $scry->isOpen($poll)) {
            $url = Util::addParameter(Horde::applicationUrl('conclude.php'), 'poll_id', $poll_id);
            $url = Util::addParameter($url, 'url', Horde::selfUrl(true, false, true));
            $polls[$poll_id]['links'][] = Horde::link($url, _("Conclude"), 'smallheader') . _("Conclude") . '</a>';
        }

        /* Edit poll link, only if editable and with perms. */
        if (Scry::checkPerms() && $scry->isEditable($poll_id, $votes[$poll_id])) {
            $url = Util::addParameter(Horde::applicationUrl('edit.php'), 'poll_id', $poll_id);
            $polls[$poll_id]['links'][] = Horde::link($url, _("Edit"), 'smallheader') . _("Edit") . '</a>';
        }

        /* Delete poll link. */
        if (Scry::checkPerms()) {
            $url = Util::addParameter(Horde::applicationUrl('delete.php'), 'poll_id', $poll_id);
            $url = Util::addParameter($url, 'url', Horde::selfUrl(true, false, true));
            $polls[$poll_id]['links'][] = Horde::link($url, _("Delete"), 'smallheader') . _("Delete") . '</a>';

            /* Toggle state poll link. */
            $url = Util::addParameter(Horde::applicationUrl('edit.php'), array('poll_id' => $poll_id, 'url' => Horde::selfURL(true, false, true)));
            if (!$poll['poll_enabled']) {
                $url = Util::addParameter($url, 'enabled', 1);
                $polls[$poll_id]['links'][] = Horde::link($url, _("Enable"), 'smallheader') . _("Enable") . '</a>';
            } else {
                $url = Util::addParameter($url, 'enabled', 0);
                $polls[$poll_id]['links'][] = Horde::link($url, _("Disable"), 'smallheader') . _("Disable") . '</a>';
            }
        }

        $polls[$poll_id]['view_url'] = Util::addParameter(Horde::applicationUrl('view.php'), 'poll_id', $poll_id);
        $polls[$poll_id]['results'] = Scry::getResults($votes[$poll_id]);
    }
}

/* Build the tabs for switching between poll lists. */
$pollsurl = Horde::applicationUrl('polls.php');
$tabs = &new Horde_UI_Tabs('show', $vars);
$tabs->addTab(_("Open Polls"), $pollsurl, 'open');
$tabs->addTab(_("All Polls"), $pollsurl, 'all');
$tabs->addTab(_("Closed Polls"), $pollsurl, 'closed');
if (Auth::isAdmin('scry:admin')) {
    $tabs->addTab(_("Disabled Polls"), $pollsurl, 'disabled');
}

/* Set the list of polls to the template. */
$template = &new Horde_Template();
$template->set('polls', $polls);
$template->set('menu', Scry::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));
$template->set('tabs', $tabs->render());

/* Construct pager. */
if ($polls_num > $perpage) {
    require_once 'Horde/UI/Pager.php';
    $config = array('num' => $polls_num,
                    'url' => 'polls.php',
                    'perpage' => $perpage);
    $pager = &new Horde_UI_Pager('page', $vars, $config);
    $template->set('pager', $pager->render());
} else {
    $template->set('pager', '');
}

$title = _("Poll List");
require SCRY_TEMPLATES . '/common-header.inc';
echo $template->fetch(SCRY_TEMPLATES . '/polls/polls.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
