<?php
/**
 * The Scry script to view a poll.
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * $Horde: scry/view.php,v 1.34 2005/01/03 14:35:27 jan Exp $
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('SCRY_BASE', dirname(__FILE__));
require_once SCRY_BASE . '/lib/base.php';

$poll_id = Util::getFormData('poll_id');
if (empty($poll_id)) {
    $notification->push(_("No poll selected."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('polls.php', true));
    exit;
}

$poll = $scry->getPolls($poll_id);
$poll['votes'] = $scry->getVotes($poll_id);

if (Auth::isAdmin('scry:admin')) {
    $respondents = $scry->getWhoChoseWhat($poll_id);
    $results = Scry::getExtendedResults($poll['votes'], $respondents);
} else {
    $results = Scry::getResults($poll['votes']);
}

/* Set up the template action links. */
$actions = array();
if ($scry->isVotable($poll)) {
    $url = Util::addParameter(Horde::applicationUrl('vote.php'), 'poll_id', $poll_id);
    $url = Util::addParameter($url, 'url', Horde::selfUrl(true, false, true));
    $actions[] = Horde::link($url, _("Vote"), 'smallheader') . _("Vote") . '</a>';
}
if (Scry::checkPerms() && $scry->isOpen($poll)) {
    $url = Util::addParameter(Horde::applicationUrl('conclude.php'), 'poll_id', $poll_id);
    $url = Util::addParameter($url, 'url', Horde::selfUrl(true, false, true));
    $actions[] = Horde::link($url, _("Conclude"), 'smallheader') . _("Conclude") . '</a>';
}
if ($scry->isEditable($poll_id)) {
    $actions[] = Horde::link(Util::addParameter(Horde::url('edit.php'), 'poll_id', $poll_id), _("Edit"), 'smallheader') . _("Edit") . '</a>';
}
if (Scry::checkPerms()) {
    $url = Util::addParameter(Horde::url('delete.php'), 'poll_id', $poll_id);
    $url = Util::addParameter($url, 'url', Horde::selfUrl(true, false, true));
    $actions[] = Horde::link($url, _("Delete"), 'smallheader') . _("Delete") . '</a>';

    $url = Horde::applicationUrl(Util::addParameter('edit.php', array('url' => Horde::selfUrl(true, false, true),
                                                                      'poll_id' => $poll_id)));
    if (!empty($poll['poll_enabled'])) {
        $actions[] = Horde::link(Util::addParameter($url, 'enabled', 0), _("Disable"), 'smallheader') . _("Disable") . '</a>';
    } else {
        $actions[] = Horde::link(Util::addParameter($url, 'enabled', 1), _("Enable"), 'smallheader') . _("Enable") . '</a>';
    }
}

list($poll['poll_enabled'], $poll['poll_enabled_class']) = Scry::getStatusLabel($poll['poll_enabled'], $poll['poll_start'], $poll['poll_end']);
$poll['poll_start'] = Scry::getDate($poll['poll_start'], _("No Start Date"));
$poll['poll_end'] = Scry::getDate($poll['poll_end'], _("No End Date"));

/* Add Agora comments/threads if enabled. */
$comments = false;
$threads = false;
if (($conf['comments']['allow'] == 'all' || ($conf['comments']['allow'] == 'authenticated' && Auth::getAuth())) &&
    $registry->hasMethod('forums/renderThreads') &&
    $registry->hasMethod('forums/postMessage')) {
    $params = array('title' => _("Add a comment"));
    if ($parent = Util::getFormData('parent')) {
        $params['message_parent_id'] = $parent;
    }
    $comments = $registry->call('forums/postMessage', array('scry', $poll_id, 'commentCallback', $params));
    $threads = $registry->call('forums/renderThreads', array($poll_id, true, 'scry', str_replace('%25', '%', Util::addParameter(Horde::selfUrl(true), 'parent', '%p'))));
    if (is_a($threads, 'PEAR_Error')) {
        $threads = $threads->getMessage();
    }
    if (is_a($comments, 'PEAR_Error')) {
        $comments = $comments->getMessage();
    }
}

$template = &new Horde_Template();
$template->set('actions', $actions, true);
$template->set('menu', Scry::getMenu('string'));
$template->set('poll', $poll);
$template->set('results', $results);
$template->set('threads', $threads, true);
$template->set('comments', $comments, true);
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));

$title = $poll['poll_title'];
require SCRY_TEMPLATES . '/common-header.inc';
echo $template->fetch(SCRY_TEMPLATES . '/view/view.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
