<?php
/**
 * The Scry script to create present a poll voting
 *
 * Copyright 2003-2005 Marko Djukic <marko@oblo.com>
 *
 * $Horde: scry/vote.php,v 1.31 2005/01/03 14:35:27 jan Exp $
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('SCRY_BASE', dirname(__FILE__));
require_once SCRY_BASE . '/lib/base.php';
require_once SCRY_BASE . '/lib/Vote.php';

/* Fetch the requested poll and check if available. */
$poll_id = Util::getFormData('poll_id');
$poll = $scry->getPolls($poll_id);
if (!$poll) {
    $notification->push(_("This poll is not available."), 'horde.warning');
    header('Location: ' . Horde::applicationUrl('polls.php', true));
    exit;
}

/* Set up the form variables. */
require_once 'Horde/Variables.php';
$vars = &Variables::getDefaultVariables();
$form_submitted = ($vars->get('formname') == 'voteform');

/* Check if form has been submitted for a voteable poll. */
$is_votable = $scry->isVotable($poll);
if ($form_submitted && $is_votable) {
    $formvote = &Horde_Form::singleton('VoteForm', $vars);
    if ($formvote->validate($vars)) {
        $formvote->getInfo($vars, $info);
        $results = $scry->logVote($info);
        if (is_a($results, 'PEAR_Error')) {
            $notification->push(sprintf(_("Voting failed. %s"), $results->getMessage()), 'horde.error');
        } else {
            $notification->push(_("Thank you for your vote."), 'horde.success');
            $url = Horde::selfURL(true, false, true);
            header('Location: ' . $url);
            exit;
        }
    }
}

/* Set up the renderer. */
$vars = &new Variables($poll);
$formvote = &Horde_Form::singleton('VoteForm', $vars);
$renderer = &$formvote->getRenderer();

if (!$is_votable) {
    $formvote->addVariable('', 'results', 'html', false);

    /* Get the current voting results */
    $votes = $scry->getVotes($poll_id);
    $vars->set('results', Scry::getResults($votes));
    $main = Util::bufferOutput(array($formvote, 'renderInactive'), $renderer, $vars);
} else {
    $main = Util::bufferOutput(array($formvote, 'renderActive'), $renderer, $vars, Horde::selfURL(true), 'post');
}

/* Set the template. */
$template = &new Horde_Template();
$template->set('main', $main);
$template->set('menu', Scry::getMenu('string'));
$template->set('notify', Util::bufferOutput(array($notification, 'notify')));

$title = sprintf(_("Vote on %s"), $poll['poll_title']);
require SCRY_TEMPLATES . '/common-header.inc';
echo $template->fetch(SCRY_TEMPLATES . '/main/main.html');
require $registry->get('templates', 'horde') . '/common-footer.inc';
